@@ -124,13 +124,12 @@ def apply_python_control(level: str, control: Control,  # noqa: C901
         arguments = substitute(arguments, configuration, secrets)
 
     sig = inspect.signature(func)
-    if "secrets" in provider and "secrets" in sig.parameters:
-        arguments["secrets"] = {}
-        for s in provider["secrets"]:
-            arguments["secrets"].update(secrets.get(s, {}).copy())
+
+    if "secrets" in sig.parameters:
+        arguments["secrets"] = secrets
 
     if "configuration" in sig.parameters:
-        arguments["configuration"] = configuration.copy()
+        arguments["configuration"] = configuration
 
     if "state" in sig.parameters:
         arguments["state"] = state
